/***********************************************************************
 * Copyright (c) 2010-2013 by SAS Institute Inc., Cary, NC, USA.
 *
 * NAME: rdcube_generate_table_code.sas
 *
 * PURPOSE: Build data sets to use when building olap cubes and infomaps
 *
 * PARAMETERS:
 *
 ***********************************************************************/

%macro rdcube_generate_table_code;
   %rdcube_delete_physical_data_sets;
   %rdcube_delete_metadata_data_sets;
   %rdcube_create_physical_data_sets;
   %rdcube_create_metadata_data_sets;
%mend rdcube_generate_table_code;

%macro rdcube_delete_physical_data_sets;
   data _NULL_;
      file MyFile mod;
      set &results_libref..ccpages end=last;
   if _N_ = 1 then do;
      put " ";
      put "/* delete physical data sets that cube will need */";
      put 'PROC DATASETS LIBRARY=&metaref NOLIST NODETAILS;';
      put "     DELETE";
      %if &write_detail_data %then %do;
          put "            &results_name._d";
          %end;
   end;
   put "            &results_name._" pagenumber;
   if last then do;
      put "            ;";
      put "   QUIT;";
      %RDCUBE_SET_RSKERR;
      %RDCUBE_CHECK_RSKERR( 'rdcube_5' );
   end;
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;
%mend rdcube_delete_physical_data_sets;

%macro rdcube_delete_metadata_data_sets;
   data _NULL_;
      file MyFile mod;
      set &results_libref..ccpages end=last;
   if _N_ = 1 then do;
      put " ";
      put "/* delete metadata references to physical data sets for cube */";
      put "PROC METALIB;";
      put "   OMR ( LIBRARY='&out_metalib' );";
      put "   UPDATE_RULE=(DELETE);";
      put "   FOLDER='&out_folder';";
      put "   SELECT (";
      %if &write_detail_data %then %do;
         put "            &results_name._d";
         %end;
   end;
   put "            &results_name._" pagenumber;
   if last then do;
      put "            );";
      put "   RUN;";
      %RDCUBE_SET_RSKERR;
      %RDCUBE_CHECK_RSKERR( 'rdcube_6' );
   end;
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;
%mend rdcube_delete_metadata_data_sets;

%macro rdcube_create_physical_data_sets;
   data _NULL_;
      file MyFile mod;
      %if %upcase(&analysis_type) ne SUMMARY %then %do;
         set &results_libref.._rd_analysis(where=(AnalysisNumber = &analysis_number));
      %end;
      put " ";
      %if &write_detail_data %then %do;
         put "/* create data sets for cube drillthrough table */";
         put 'DATA &metaref' "..&results_name._d;";
         put "   SET &drillthrough_table;";
         put "   LENGTH top $1;";
         put '   LABEL top="Portfolio Level Results";';
         put '   top="+";';
         %if %upcase(&analysis_type) ne SUMMARY %then %do;
            put '   DO sim_item_counter = 1 TO ' NumItems ';';
            put '      OUTPUT;';
            put '      END;';
            %end;
         put "   RUN;";
         put " ";
         %end;
   run;
   %if %upcase(&analysis_type) ne SUMMARY %then %do;
      data _NULL_;
         file MyFile mod;
         set &results_libref..Dimension_Information(where=(Analysis_Number = &analysis_number));
         put "DATA work._rdcube_&analysis_number._temp;";
         put "   SET &data_table(where=(AnalysisNumber = &analysis_number));";
         if Category_Flag = 1 then do; 
            put "   LENGTH SimulationCategory1 $ 100;";
         end;
         if Horizon_Flag = 1 then do; 
            put "   LENGTH SimulationHorizon1 $ 100;";
         end;
         if Category_Flag = 1 then do; 
            put "   IF SimulationMode EQ 'Unconditional' THEN";
            put "      SimulationCategory1 = SimulationMode;";
            put "   ELSE";
            put "      SimulationCategory1 = strip(SimulationMode) || ' on ' || strip(SimulationCategory);";
         end;
         if Horizon_Flag = 1 then do; 
            put "   SimulationHorizon1 = strip(SimulationHorizon);";
/*put "   SimulationHorizon1 = strip(SimulationHorizon) || ' (' || _date_ || ');'";*/
         end;
         put "   RUN;";
         put " ";
         put 'DATA &metaref' "..&results_name._d;";
         put 'MERGE &metaref' "..&results_name._d work._rdcube_&analysis_number._temp(KEEP= ";
         if Outvar_Flag = 1 then do; 
            %if %upcase(&analysis_type) eq SIMULATION %then %do;
               put "   SimulationPart";
            %end;
            %else %do;
               put "   OutputVariable";
            %end;
         end;
         if Category_Flag = 1 then do; 
            put "   SimulationCategory1";
         end;
         if Horizon_Flag = 1 then do; 
            put "   SimulationHorizon1";
         end;
         put "   );";
         put "   RUN;";
         put " ";
      run;
      %rdcube_check_syserr;
      %if &syserr ne 0 %then %return;
      %end;
   data _NULL_;
      file MyFile mod;
      set &results_libref..ccpages end=last;
   if _N_ = 1 then do;
      put "/* create data sets for cube aggregations */";
      put "DATA";
   end;
   put '   &metaref' "..&results_name._" pagenumber;
   put "          (DROP = pagen " droplist " )";
   if last then do;
      put '          ;';
      put ' ';
      put "   SET &data_table"
          %if &analysis_number NE 0 %then %do; "(where=(AnalysisNumber = &analysis_number))" %end;
          '   ;';
      put ' ';
   end;
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   %if %upcase(&analysis_type) ne SUMMARY %then %do;
      data _NULL_;
         file MyFile mod;
         set &results_libref..Dimension_Information(where=(Analysis_Number = &analysis_number));
      if Category_Flag = 1 then do; 
         put "   LENGTH SimulationCategory1 $ 100;";
      end;
      if Horizon_Flag = 1 then do; 
         put "   LENGTH SimulationHorizon1 $ 100;";
      end;
      if Category_Flag = 1 then do; 
         put "   IF SimulationMode EQ 'Unconditional' THEN";
         put "      SimulationCategory1 = SimulationMode;";
         put "   ELSE";
         put "      SimulationCategory1 = strip(SimulationMode) || ' on ' || strip(SimulationCategory);";
      end;
      if Horizon_Flag = 1 then do; 
         put "   SimulationHorizon1 = strip(SimulationHorizon);";
/*put "   SimulationHorizon1 = strip(SimulationHorizon) || ' (' || _date_ || ');'";*/
      end;
      put " ";
      run;
      %end;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   %if %upcase(&analysis_type) eq SENSITIVITY %then %do;
      data _NULL_;
         file MyFile mod;
         set &results_libref..Dimension_Information(where=(Analysis_Number = &analysis_number));
      put "   LENGTH _TYPE1_ $ 41;";
      put "   IF LENGTH(Name) EQ 1 THEN";
      put "      _TYPE1_ =  _TYPE_;";
      put "   ELSE";
      put '      _TYPE1_ = "GAMMA on " || Name;';
      run;
      %end;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   data _NULL_;
      file MyFile mod;
      set &results_libref..ccvars end=last;
      label1 = tranwrd(label, "'", "''");
   put "   LABEL "  class_var_id " = '" label1 "';";
   if class_var_id ne class_var then
      put "   RENAME " class_var_id " = "  class_var ";";
   if last then do;
      put "   LENGTH TOP $1 ;";
      put "   LABEL TOP = 'Top level aggregation' ;";
      put "   TOP = '+' ;";
   end;
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   data _NULL_;
      file MyFile mod;
      set &results_libref..ccvars end=last;
   if _N_ = 1 then do;
      put " ";
      put "   pagen = 0;";
   end;
   if upcase(type) eq "CHAR" then
      put "   IF " class_var_id ' NE "+" THEN';
   else
      put "   IF " class_var_id ' NE ._ THEN';
   page_number = 2**(_n_-1);
   put "      pagen = pagen + " page_number ";";
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   data _NULL_;
      file MyFile mod;
      set &results_libref..ccpages end=last;
   if _N_ = 1 then do;
      put " ";
   end;
   put "   IF pagen = " pagenumber " THEN";
   put '      OUTPUT &metaref' "..&results_name._" pagenumber ";";
   if last then do;
      put "   RUN;";
      %RDCUBE_SET_RSKERR;
      %RDCUBE_CHECK_RSKERR( 'rdcube_7' );
   end;
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;
%mend rdcube_create_physical_data_sets;

%macro rdcube_create_metadata_data_sets;
   data _NULL_;
      file MyFile mod;
      set &results_libref..ccpages end=last;
   if _N_ = 1 then do;
      put " ";
      put "/* register data sets for cube */";
      put "PROC METALIB;";
      put "   OMR ( LIBRARY='&out_metalib' );";
      put "   FOLDER='&out_folder';";
      put "   SELECT (";
      %if &write_detail_data %then %do;
         put "            &results_name._d";
         %end;
   end;
   put "            &results_name._" pagenumber;
   if last then do;
      put "            );";
      put "   RUN;";
      %RDCUBE_SET_RSKERR;
      %RDCUBE_CHECK_RSKERR( 'rdcube_8' );
   end;
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;
%mend rdcube_create_metadata_data_sets;
